---@type Tinkr
local Tinkr,
---@class Bastion
Bastion = ...

---@class Bastion.ObjectManager
---@field _lists table<string, { list: Bastion.List, cb: fun(object: TinkrObjectReference): false | Bastion.Unit }>
---@field afflicted Bastion.List
---@field enemies Bastion.List
---@field critters Bastion.List
---@field friends Bastion.List
---@field activeEnemies Bastion.List
---@field explosives Bastion.List
---@field incorporeal Bastion.List
---@field others Bastion.List
local ObjectManager = {}
ObjectManager.__index = ObjectManager

function ObjectManager:New()
    local self = setmetatable({}, ObjectManager)

    self._lists = {}

    self.enemies = Bastion.List:New()
    self.critters = Bastion.List:New()
    self.friends = Bastion.List:New()
    self.activeEnemies = Bastion.List:New()
    self.explosives = Bastion.List:New()
    self.incorporeal = Bastion.List:New()
    self.afflicted = Bastion.List:New()
    self.others = Bastion.List:New()

    return self
end

-- Register a custom list with a callback
---@param name string
---@param cb fun(object: TinkrObjectReference): false | Bastion.Unit
---@return Bastion.List | false
function ObjectManager:RegisterList(name, cb)
    if self._lists[name] then
        return false
    end

    self._lists[name] = {
        list = Bastion.List:New(),
        cb = cb,
    }

    return self._lists[name].list
end

-- reset custom lists
---@return nil
function ObjectManager:ResetLists()
    for _, list in pairs(self._lists) do
        list.list:clear()
    end
end

-- Refresh custom lists
---@param object table
---@return nil
function ObjectManager:EnumLists(object)
    for _, list in pairs(self._lists) do
        local r = list.cb(object)
        if r then
            list.list:push(r)
        end
    end
end

-- Get a list
---@param name string
---@return Bastion.List
function ObjectManager:GetList(name)
    return self._lists[name].list
end

-- Refresh all lists
---@return nil
function ObjectManager:Refresh()
    self.afflicted:clear()
    self.enemies:clear()
    self.critters:clear()
    self.friends:clear()
    self.activeEnemies:clear()
    self.explosives:clear()
    self.incorporeal:clear()
    self.others:clear()
    self:ResetLists()

    local objects = Objects()

    for _, object in pairs(objects) do
        self:EnumLists(object)
        local objectType = ObjectType(object)
        if ({ [5] = true, [6] = true, [7] = true })[objectType] then
            local objectGUID = ObjectGUID(object)
            if objectGUID then
                local unit = Bastion.UnitManager:GetObject(objectGUID)
                if not unit then
                    unit = Bastion.Unit:New(object)
                    Bastion.UnitManager:SetObject(unit)
                end

                local creatureType = ObjectCreatureType(object)
                if objectType == 5 and creatureType == 8 then
                    self.critters:push(unit)
                elseif unit:GetID() == 204560 then
                    self.incorporeal:push(unit)
                elseif unit:GetID() == 204773 then
                    self.afflicted:push(unit)
                elseif unit:GetID() == 120651 then
                    self.explosives:push(unit)
                elseif unit:IsPlayer() and (unit:IsInPartyOrRaid() or unit == Bastion.UnitManager["player"]) then
                    self.friends:push(unit)
                elseif unit:IsEnemy() then
                    self.enemies:push(unit)
                    if unit:InCombatOdds() > 80 or unit:IsAffectingCombat() then
                        self.activeEnemies:push(unit)
                    end
                else
                    self.others:push(unit)
                end
            end
        end
    end
end

Bastion.ObjectManager = ObjectManager:New()
